@@ -1,26 +1,48 @@
+"""
+Environment definitions to create an environment for a deployment.
+"""
+
 from __future__ import annotations
 
 import platform
 from dataclasses import dataclass, field
-from typing import Optional, Dict, List
+from typing import Dict, List, Optional, cast
 
 from .types import (
-    RuntimeContextRepresentation,
-    EnvironmentDefinitionRepresentation,
-    ShellExecutionRepresentation,
     BuildContextRepresentation,
     DockerImageRepresentation,
+    EnvironmentDefinitionRepresentation,
+    RuntimeContextRepresentation,
+    ShellExecutionRepresentation,
 )
 
-
 # TODO: make classes frozen to make hash stable
 
 
 @dataclass
 class RuntimeContext:
+    """
+    Stores a runtime context for a Deployment, such as port mapping or environment variables, that would be available during deployment
+
+    :param ports_mapping: map of ports to be mapped to the host
+    :param environment_variables: map of environment variables to be set
+    :param additional_arguments: list of additional arguments to be passed to the runtime
+    """
+
     ports_mapping: Dict[int, int] = field(default_factory=dict)
+    """
+    Desired port mapping (for all protocols)
+    """
+
     environment_variables: Dict[str, str] = field(default_factory=dict)
+    """
+    Desired values of environment variables
+    """
+
     additional_arguments: List[str] = field(default_factory=list)
+    """
+    Additional arguments (could be interpreted by runtimes)
+    """
 
     def __json__(self) -> RuntimeContextRepresentation:
         return {
@@ -31,6 +53,13 @@ class RuntimeContext:
 
     @classmethod
     def from_json(cls, data: RuntimeContextRepresentation) -> RuntimeContext:
+        """
+        Creates RuntimeContext from JSON representation.
+
+        :param data: JSON representation of RuntimeContext
+        :return: Deserialized RuntimeContext
+        """
+
         ports_mapping = data["ports_mapping"]
         environment_variables = data["environment_variables"]
         additional_arguments = data["additional_arguments"]
@@ -43,7 +72,21 @@ class RuntimeContext:
 
 @dataclass
 class EnvironmentDefinition:
+    """
+    A base class for all Environment Definitions.
+
+    :param commands: commands that should be executed to create an environment
+    """
+
     commands: List[str] = field(default_factory=list)
+    """
+    A list of commands that should be executed to create an environment.
+    """
+
+    runtime_context: RuntimeContext = field(default_factory=RuntimeContext)
+    """
+    Runtime context for this environment definition
+    """
 
     def __hash__(self) -> int:
         return hash(tuple(self.commands))
@@ -52,6 +95,13 @@ class EnvironmentDefinition:
     def from_json(
         cls, data: EnvironmentDefinitionRepresentation
     ) -> EnvironmentDefinition:
+        """
+        Creates EnvironmentDefinition from JSON representation.
+
+        :param data: JSON representation of EnvironmentDefinition
+        :return: Deserialized EnvironmentDefinition
+        """
+
         instance = cls.__new__(cls)
         instance.commands = data["commands"]
         return instance
@@ -59,28 +109,30 @@ class EnvironmentDefinition:
     def __json__(self) -> EnvironmentDefinitionRepresentation:
         return {
             "commands": self.commands,
+            "runtime_context": self.runtime_context.__json__(),
         }
 
 
 @dataclass
 class ShellExecution(EnvironmentDefinition):
     """
-    This class represents Environment Definition that's created by executing shell commands.
+    This class represents Environment Definition that's created by executing commands in OS's shell.
+
+    :param runtime_context: runtime context for this environment definition
     """
 
-    runtime_context: RuntimeContext = field(default_factory=RuntimeContext)
-
-    def __json__(self) -> ShellExecutionRepresentation:
-        return {
-            "commands": self.commands,
-            "runtime_context": self.runtime_context.__json__(),
-        }
-
     @classmethod
     def from_json(cls, _data: EnvironmentDefinitionRepresentation) -> ShellExecution:
+        """
+        Creates ShellExecution from JSON representation.
+
+        :param _data: JSON representation of ShellExecution
+        :return: Deserialized ShellExecution
+        """
+
         if "runtime_context" not in _data:
             raise ValueError("runtime_context is missing")
-        data: ShellExecutionRepresentation = _data  # type: ignore
+        data = cast(ShellExecutionRepresentation, _data)
         instance = cls.__new__(cls)
         instance.commands = data["commands"]
         instance.runtime_context = RuntimeContext.from_json(data["runtime_context"])
@@ -89,19 +141,31 @@ class ShellExecution(EnvironmentDefinition):
 
 @dataclass(frozen=True)
 class BuildContext:
-    """ """
+    """
+    Stores a build context for a Deployment, such as Python version or cloudpickle version, used during deployment
+
+    :param python_version: User's Python version
+    :param cloudpickle_version: User's cloudpickle version
+    """
+
+    python_version: str = field(default_factory=platform.python_version)
+    """
+    User's Python version
+    """
 
-    python_version: str = field(default_factory=lambda: platform.python_version())
     cloudpickle_version: Optional[str] = field(
         default_factory=lambda: BuildContext._get_cloudpickle_version()
     )
+    """
+    User's cloudpickle version
+    """
 
     @staticmethod
     def _get_cloudpickle_version() -> Optional[str]:
         try:
             import cloudpickle
 
-            return cloudpickle.__version__  # type: ignore
+            return cast(str, cloudpickle.__version__)
         except ImportError:
             return None
 
@@ -113,6 +177,13 @@ class BuildContext:
 
     @classmethod
     def from_json(cls, data: BuildContextRepresentation) -> BuildContext:
+        """
+        Creates BuildContext from JSON representation.
+
+        :param data: JSON representation of BuildContext
+        :return: Deserialized BuildContext
+        """
+
         python_version = data["python_version"]
         cloudpickle_version = data["cloudpickle_version"]
         return cls(
@@ -123,13 +194,28 @@ class BuildContext:
 @dataclass
 class DockerImage(EnvironmentDefinition):
     """
-    This class represents Environment Definition that is created by using a Docker image.
-    If image name is not provided, then it would be created automatically
+    | This class represents Environment Definition that is created by using a Docker image.
+    | If image name is not provided, then it would be created automatically
+
+    :param image: Docker image name
+    :param build_context: build context for this environment definition
+    :param runtime_context: runtime context for this environment definition
     """
 
     image: Optional[str] = None
+    """
+    Docker image name
+    """
+
     build_context: BuildContext = field(default_factory=BuildContext)
+    """
+    Build context for this environment definition
+    """
+
     runtime_context: RuntimeContext = field(default_factory=RuntimeContext)
+    """
+    Runtime context for this environment definition
+    """
 
     def __hash__(self) -> int:
         if self.image:
@@ -153,12 +239,25 @@ class DockerImage(EnvironmentDefinition):
 
     @classmethod
     def from_json(cls, _data: EnvironmentDefinitionRepresentation) -> DockerImage:
+        """
+        Creates DockerImage from JSON representation.
+
+        :param _data: JSON representation of DockerImage
+        :return: Deserialized DockerImage
+        """
+
         if "image" not in _data:
             raise ValueError("image is missing")
-        data: DockerImageRepresentation = _data  # type: ignore
+        data = cast(DockerImageRepresentation, _data)
         instance = cls.__new__(cls)
         instance.commands = data["commands"]
         instance.image = data["image"]
         instance.build_context = BuildContext.from_json(data["build_context"])
         instance.runtime_context = RuntimeContext.from_json(data["runtime_context"])
         return instance
+
+
+_available_environment_definitions = {
+    ShellExecution.__name__: ShellExecution,
+    DockerImage.__name__: DockerImage,
+}
